#!/usr/bin/env python3

import argparse
import gzip
import os
import numpy as np
from multiprocessing import Pool
from rdkit import Chem

count = 0
done = False
args = None
records_per_process = 0


def process_file(file_path):
    global args, count, done, records_per_process
    if done:
        return
    outfile_name = "%s.txt" % file_path.split(".")[0]
    outfile = open(os.path.join(args.output, outfile_name), "w")
    print("file: %s" % file_path)
    gzfile = gzip.open(os.path.join(args.input, file_path))
    with Chem.ForwardSDMolSupplier(gzfile) as reader:
        for m in reader:
            if m is None:
                continue
            p = Chem.RDKFingerprint(m)
            if p.GetNumOnBits() == 0:
                print("WARNING: invalid fingerprint")
                continue
            p = np.packbits(p)
            outfile.write("%s\n" % ",".join([str(x) for x in p]))
            count += 1
            if count >= records_per_process:
                outfile.close()
                done = True
                break
    print("count: %d" % count)


def main(input_dir, parallelism):
    files = sorted(os.listdir(input_dir))
    with Pool(processes=parallelism) as pool:
        pool.map(process_file, files)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument("--output", type=str, help="output dir", default="./out")
    parser.add_argument(
        "--input",
        type=str,
        help="path to directory containing sdf files",
        default="sdf",
    )
    parser.add_argument("-c", type=int, help="total # of records", default=10000)
    parser.add_argument("-p", type=int, help="num of processes", default=10)
    args = parser.parse_args()

    records_per_process = (args.c + args.p - 1) / args.p
    main(args.input, args.p)
