#!/usr/bin/env python3

import argparse
import os

import h5py
import numpy as np


def parse_line(line):
    return np.array([int(x) for x in line.split(",")], dtype="uint8")


def load_dataset(path):
    with open(path) as f:
        data = np.array([parse_line(line) for line in f])
    return data


def parse_gt_line(line):
    fields = line.split(",")
    neighbors = [int(x) for x in fields[0:len(fields):2]]
    dists = [float(x) for x in fields[1:len(fields):2]]
    return neighbors, dists


def load_groundtruth(path):
    file_list = sorted([p for p in os.listdir(path) if p.startswith("gt.")])
    neighbors = []
    dists = []
    for p in file_list:
        with open(os.path.join(path, p)) as f:
            for line in f:
                n, d = parse_gt_line(line)
                neighbors.append(n)
                dists.append(d)
    return neighbors, dists


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument("--input", type=str, help="input dir", default="sdf")
    parser.add_argument(
        "--output", type=str, help="output hdf5", default="./pubchem.hdf5"
    )
    args = parser.parse_args()

    train_set = load_dataset(os.path.join(args.input, "train.txt"))
    print("train set size: %s" % str(train_set.shape))
    test_set = load_dataset(os.path.join(args.input, "test.txt"))
    print("test set size: %s" % str(test_set.shape))

    neighbors, dists = load_groundtruth(args.input)
    print("neighbors: %s" % len(neighbors))
    print("dists: %s" % len(dists))

    f = h5py.File(args.output, "w")
    f.attrs["type"] = "dense"
    f.attrs["distance"] = "jaccard"
    f.attrs["dimension"] = train_set.shape[1] * 8 
    f.attrs["point_type"] = "bitmap"

    f.create_dataset("train", data=train_set, dtype=train_set.dtype)
    f.create_dataset("test", data=test_set, dtype=test_set.dtype)
    f.create_dataset("neighbors", data=neighbors)
    f.create_dataset("distances", data=dists)
